# -*- coding: utf-8  -*-
#
# Copyright (C) 2011-2014 Ben Kurtovic <ben.kurtovic@gmail.com>
# See the LICENSE file for details.

from __future__ import print_function

import os

from colorama import Fore, Style
from git import Repo, exc

__all__ = ["update_bookmarks", "update_directories"]

BOLD = Style.BRIGHT
RED = Fore.RED + BOLD
GREEN = Fore.GREEN + BOLD
BLUE = Fore.BLUE + BOLD
RESET = Style.RESET_ALL

INDENT1 = " " * 3
INDENT2 = " " * 7

def _update_repository(repo, repo_name):
    """Update a single git repository by pulling from the remote."""
    print(INDENT1, BOLD + repo_name + ":")

    try:
        # Check if there is anything to pull, but don't do it yet:
        dry_fetch = _exec_shell("git fetch --dry-run")
    except subprocess.CalledProcessError:
        print(INDENT2, RED + "Error:" + RESET, "cannot fetch;",
              "do you have a remote repository configured correctly?")
        return

    try:
        last_commit = _exec_shell("git log -n 1 --pretty=\"%ar\"")
    except subprocess.CalledProcessError:
        last_commit = "never"  # Couldn't get a log, so no commits

    if not dry_fetch:  # No new changes to pull
        print(INDENT2, BLUE + "No new changes." + RESET,
              "Last commit was {0}.".format(last_commit))

    else:  # Stuff has happened!
        print(INDENT2, "There are new changes upstream...")
        status = _exec_shell("git status")

        if status.endswith("nothing to commit, working directory clean"):
            print(INDENT2, GREEN + "Pulling new changes...")
            result = _exec_shell("git pull")
            if last_commit == "never":
                print(INDENT2, "The following changes have been made:")
            else:
                print(INDENT2, "The following changes have been made since",
                      last_commit + ":")
            print(result)

        else:
            print(INDENT2, RED + "Warning:" + RESET,
                  "you have uncommitted changes in this repository!")
            print(INDENT2, "Ignoring.")

def _update_subdirectories(dir_path, dir_name, dir_long_name):
    """Update all subdirectories that are git repos in a given directory."""
    repos = []
    for item in os.listdir(dir_path):
        try:
            repo = Repo(os.path.join(dir_path, item))
        except (exc.InvalidGitRepositoryError, exc.NoSuchPathError):
            continue
        repos.append((repo, os.path.join(dir_name, item)))

    if len(repos) == 1:
        print(dir_long_name.capitalize(), "contains 1 git repository:")
    else:
        print(dir_long_name.capitalize(),
              "contains {0} git repositories:".format(len(repos)))

    for repo_path, repo_name in sorted(repos):
        _update_repository(repo_path, repo_name)

def _update_directory(dir_path, dir_name, is_bookmark=False):
    """Update a particular directory.

    Determine whether the directory is a git repo on its own, a directory of
    git repositories, or something invalid. If the first, update the single
    repository; if the second, update all repositories contained within; if the
    third, print an error.
    """
    dir_type = "bookmark" if is_bookmark else "directory"
    dir_long_name = dir_type + ' "' + BOLD + dir_path + RESET + '"'

    try:
        repo = Repo(dir_path)
    except exc.NoSuchPathError:
        print(RED + "Error:" + RESET, dir_long_name, "doesn't exist!")
    except exc.InvalidGitRepositoryError:
        if os.path.isdir(dir_path):
            _update_subdirectories(dir_path, dir_name, dir_long_name)
        else:
            print(RED + "Error:" + RESET, dir_long_name, "isn't a repository!")
    else:
        print(dir_long_name.capitalize(), "is a git repository:")
        _update_repository(repo, dir_name)

def update_bookmarks(bookmarks):
    """Loop through and update all bookmarks."""
    if bookmarks:
        for bookmark_path, bookmark_name in bookmarks:
            _update_directory(bookmark_path, bookmark_name, is_bookmark=True)
    else:
        print("You don't have any bookmarks configured! Get help with 'gitup -h'.")

def update_directories(paths):
    """Update a list of directories supplied by command arguments."""
    for path in paths:
        path = os.path.abspath(path)  # Convert relative to absolute path
        path_name = os.path.split(path)[1]  # Dir name ("x" in /path/to/x/)
        _update_directory(path, path_name, is_bookmark=False)
